from copy import deepcopy
import datetime
import json
from time import time


class RequestCreator:
    """
    A class to help build a request for Adobe Analytics API 2.0 getReport
    """

    template = {
        "globalFilters": [],
        "metricContainer": {
            "metrics": [],
            "metricFilters": [],
        },
        "settings": {
            "countRepeatInstances": True,
            "limit": 20000,
            "page": 0,
            "nonesBehavior": "exclude-nones",
        },
        "statistics": {"functions": ["col-max", "col-min"]},
        "rsid": "",
    }

    def __init__(self, request: dict = None) -> None:
        """
        Instanciate the constructor.
        Arguments:
            request : OPTIONAL : overwrite the template with the definition provided.
        """
        if request is not None:
            if '.json' in request and type(request) == str:
                with open(request,'r') as f:
                    request = json.load(f)
        self.__request = deepcopy(request) or deepcopy(self.template)
        self.__metricCount = len(self.__request["metricContainer"]["metrics"])
        self.__metricFilterCount = len(
            self.__request["metricContainer"].get("metricFilters", [])
        )
        self.__globalFiltersCount = len(self.__request["globalFilters"])
        ### Preparing some time statement.
        today = datetime.datetime.now()
        today_date_iso = today.isoformat().split("T")[0]
        ## should give '20XX-XX-XX'
        tomorrow_date_iso = (
            (today + datetime.timedelta(days=1)).isoformat().split("T")[0]
        )
        time_start = "T00:00:00.000"
        time_end = "T23:59:59.999"
        startToday_iso = today_date_iso + time_start
        endToday_iso = today_date_iso + time_end
        startMonth_iso = f"{today_date_iso[:-2]}01{time_start}"
        tomorrow_iso = tomorrow_date_iso + time_start
        next_month = today.replace(day=28) + datetime.timedelta(days=4)
        last_day_month = next_month - datetime.timedelta(days=next_month.day)
        last_day_month_date_iso = last_day_month.isoformat().split("T")[0]
        last_day_month_iso = last_day_month_date_iso + time_end
        thirty_days_prior_date_iso = (
            (today - datetime.timedelta(days=30)).isoformat().split("T")[0]
        )
        thirty_days_prior_iso = thirty_days_prior_date_iso + time_start
        seven_days_prior_iso_date = (
            (today - datetime.timedelta(days=7)).isoformat().split("T")[0]
        )
        seven_days_prior_iso = seven_days_prior_iso_date + time_start
        ### assigning predefined dates:
        self.dates = {
            "thisMonth": f"{startMonth_iso}/{last_day_month_iso}",
            "untilToday": f"{startMonth_iso}/{startToday_iso}",
            "todayIncluded": f"{startMonth_iso}/{endToday_iso}",
            "last30daysTillToday": f"{thirty_days_prior_iso}/{startToday_iso}",
            "last30daysTodayIncluded": f"{thirty_days_prior_iso}/{tomorrow_iso}",
            "last7daysTillToday": f"{seven_days_prior_iso}/{startToday_iso}",
            "last7daysTodayIncluded": f"{seven_days_prior_iso}/{endToday_iso}",
        }
        self.today = today

    def __repr__(self):
        return json.dumps(self.__request, indent=4)

    def __str__(self):
        return json.dumps(self.__request, indent=4)

    def addMetric(self, metricId: str = None) -> None:
        """
        Add a metric to the template.
        Arguments:
            metricId : REQUIRED : The metric to add
        """
        if metricId is None:
            raise ValueError("Require a metric ID")
        columnId = self.__metricCount
        addMetric = {"columnId": str(columnId), "id": metricId}
        if columnId == 0:
            addMetric["sort"] = "desc"
        self.__request["metricContainer"]["metrics"].append(addMetric)
        self.__metricCount += 1
    
    def removeMetrics(self) -> None:
        """
        Remove all metrics.
        """
        self.__request["metricContainer"]["metrics"] = []
        self.__metricCount = 0
    
    def getMetrics(self) -> list:
        """
        return a list of the metrics used
        """
        return [metric["id"] for metric in self.__request["metricContainer"]["metrics"]]
    
    def setSearch(self,clause:str=None)->None:
        """
        Add a search clause in the Analytics request.
        Arguments:
            clause : REQUIRED : String to tell what search clause to add.
                Examples: 
                "( CONTAINS 'unspecified' ) OR ( CONTAINS 'none' ) OR ( CONTAINS '' )"
                "( MATCH 'undefined' )"
                "( NOT CONTAINS 'undefined' )"
                "( BEGINS-WITH 'undefined' )"
                "( BEGINS-WITH 'undefined' ) AND ( BEGINS-WITH 'none' )"
        """
        if clause is None:
            raise ValueError("Require a clause to add to the request")
        self.__request["search"] = {
            "clause" : clause
        }


    def removeSearch(self)->None:
        """
        Remove the search associated with the request.
        """
        del self.__request["search"]

    def addMetricFilter(
        self, metricId: str = None, filterId: str = None, metricIndex: int = None
    ) -> None:
        """
        Add a filter to a metric.
        Arguments:
            metricId : REQUIRED : metric where the filter is added
            filterId : REQUIRED : The filter to add.
                when breakdown, use the following format for the value "dimension:::itemId"
            metricIndex : OPTIONAL : If used, set the filter to the metric located on that index.
        """
        if metricId is None:
            raise ValueError("Require a metric ID")
        if filterId is None:
            raise ValueError("Require a filter ID")
        filterIdCount = self.__metricFilterCount
        if filterId.startswith("s") and "@AdobeOrg" in filterId:
            filterType = "segment"
            filter = {
                "id": str(filterIdCount),
                "type": filterType,
                "segmentId": filterId,
            }
        elif filterId.startswith("20") and "/20" in filterId:
            filterType = "dateRange"
            filter = {
                "id": str(filterIdCount),
                "type": filterType,
                "dateRange": filterId,
            }
        elif ":::" in filterId:
            filterType = "breakdown"
            dimension, itemId = filterId.split(":::")
            filter = {
                "id": str(filterIdCount),
                "type": filterType,
                "dimension": dimension,
                "itemId": itemId,
            }
        else:  ### case when it is predefined segments like "All_Visits"
            filterType = "segment"
            filter = {
                "id": str(filterIdCount),
                "type": filterType,
                "segmentId": filterId,
            }
        if filterIdCount == 0:
            self.__request["metricContainer"]["metricFilters"] = [filter]
        else:
            self.__request["metricContainer"]["metricFilters"].append(filter)
        ### adding filter to the metric
        if metricIndex is None:
            for metric in self.__request["metricContainer"]["metrics"]:
                if metric["id"] == metricId:
                    if "filters" in metric.keys():
                        metric["filters"].append(str(filterIdCount))
                    else:
                        metric["filters"] = [str(filterIdCount)]
        else:
            metric = self.__request["metricContainer"]["metrics"][metricIndex]
            if "filters" in metric.keys():
                metric["filters"].append(str(filterIdCount))
            else:
                metric["filters"] = [str(filterIdCount)]
        ### incrementing the filter counter
        self.__metricFilterCount += 1

    def removeMetricFilter(self, filterId: str = None) -> None:
        """
        remove a filter from a metric
        Arguments:
            filterId : REQUIRED : The filter to add.
                when breakdown, use the following format for the value "dimension:::itemId"
        """
        found = False  ## flag
        if filterId is None:
            raise ValueError("Require a filter ID")
        if ":::" in filterId:
            filterId = filterId.split(":::")[1]
        list_index = []
        for metricFilter in self.__request["metricContainer"]["metricFilters"]:
            if filterId in str(metricFilter):
                list_index.append(metricFilter["id"])
                found = True
        ## decrementing the filter counter
        if found:
            for metricFilterId in reversed(list_index):
                del self.__request["metricContainer"]["metricFilters"][
                    int(metricFilterId)
                ]
                for metric in self.__request["metricContainer"]["metrics"]:
                    if metricFilterId in metric.get("filters", []):
                        metric["filters"].remove(metricFilterId)
                self.__metricFilterCount -= 1

    def setLimit(self, limit: int = 100) -> None:
        """
        Specific the number of element to retrieve. Default is 10.
        Arguments:
            limit : OPTIONAL : number of elements to return
        """
        self.__request["settings"]["limit"] = limit

    def setRepeatInstance(self, repeat: bool = True) -> None:
        """
        Specify if repeated instances should be counted.
        Arguments:
            repeat : OPTIONAL : True or False (True by default)
        """
        self.__request["settings"]["countRepeatInstances"] = repeat

    def setNoneBehavior(self, returnNones: bool = True) -> None:
        """
        Set the behavior of the None values in that request.
        Arguments:
            returnNones : OPTIONAL : True or False (True by default)
        """
        if returnNones:
            self.__request["settings"]["nonesBehavior"] = "return-nones"
        else:
            self.__request["settings"]["nonesBehavior"] = "exclude-nones"

    def setDimension(self, dimension: str = None) -> None:
        """
        Set the dimension to be used for reporting.
        Arguments:
            dimension : REQUIRED : the dimension to build your report on
        """
        if dimension is None:
            raise ValueError("A dimension must be passed")
        self.__request["dimension"] = dimension

    def setRSID(self, rsid: str = None) -> None:
        """
        Set the reportSuite ID to be used for the reporting.
        Arguments:
            rsid : REQUIRED : The Data View ID to be passed.
        """
        if rsid is None:
            raise ValueError("A reportSuite ID must be passed")
        self.__request["rsid"] = rsid

    def addGlobalFilter(self, filterId: str = None) -> None:
        """
        Add a global filter to the report.
        NOTE : You need to have a dateRange filter at least in the global report.
        Arguments:
            filterId : REQUIRED : The filter to add to the global filter.
                example :
                "s2120430124uf03102jd8021" -> segment
                "2020-01-01T00:00:00.000/2020-02-01T00:00:00.000" -> dateRange
        """
        if filterId.startswith("s") and "@AdobeOrg" in filterId:
            filterType = "segment"
            filter = {
                "type": filterType,
                "segmentId": filterId,
            }
        elif filterId.startswith("20") and "/20" in filterId:
            filterType = "dateRange"
            filter = {
                "type": filterType,
                "dateRange": filterId,
            }
        elif ":::" in filterId:
            filterType = "breakdown"
            dimension, itemId = filterId.split(":::")
            filter = {
                "type": filterType,
                "dimension": dimension,
                "itemId": itemId,
            }
        else:  ### case when it is predefined segments like "All_Visits"
            filterType = "segment"
            filter = {
                "type": filterType,
                "segmentId": filterId,
            }
        ### incrementing the count for globalFilter
        self.__globalFiltersCount += 1
        ### adding to the globalFilter list
        self.__request["globalFilters"].append(filter)

    def updateDateRange(
        self,
        dateRange: str = None,
        shiftingDays: int = None,
        shiftingDaysEnd: int = None,
        shiftingDaysStart: int = None,
    ) -> None:
        """
        Update the dateRange filter on the globalFilter list
        One of the 3 elements specified below is required.
        Arguments:
            dateRange : OPTIONAL : string representing the new dateRange string, such as: 2020-01-01T00:00:00.000/2020-02-01T00:00:00.000
            shiftingDays : OPTIONAL : An integer, if you want to add or remove days from the current dateRange provided. Apply to end and beginning of dateRange.
                So 2020-01-01T00:00:00.000/2020-02-01T00:00:00.000 with +2 will give 2020-01-03T00:00:00.000/2020-02-03T00:00:00.000
            shiftingDaysEnd : : OPTIONAL : An integer, if you want to add or remove days from the last part of the current dateRange. Apply only to end of the dateRange.
                So 2020-01-01T00:00:00.000/2020-02-01T00:00:00.000 with +2 will give 2020-01-01T00:00:00.000/2020-02-03T00:00:00.000
            shiftingDaysStart : OPTIONAL : An integer, if you want to add or remove days from the last first part of the current dateRange. Apply only to beginning of the dateRange.
                So 2020-01-01T00:00:00.000/2020-02-01T00:00:00.000 with +2 will give 2020-01-03T00:00:00.000/2020-02-01T00:00:00.000
        """
        pos = -1
        for index, filter in enumerate(self.__request["globalFilters"]):
            if filter["type"] == "dateRange":
                pos = index
                curDateRange = filter["dateRange"]
                start, end = curDateRange.split("/")
                start = datetime.datetime.fromisoformat(start)
                end = datetime.datetime.fromisoformat(end)
        if dateRange is not None and type(dateRange) == str:
            for index, filter in enumerate(self.__request["globalFilters"]):
                if filter["type"] == "dateRange":
                    pos = index
                    curDateRange = filter["dateRange"]
            newDef = {
                "type": "dateRange",
                "dateRange": dateRange,
            }
        if shiftingDays is not None and type(shiftingDays) == int:
            newStart = (start + datetime.timedelta(shiftingDays)).isoformat(
                timespec="milliseconds"
            )
            newEnd = (end + datetime.timedelta(shiftingDays)).isoformat(
                timespec="milliseconds"
            )
            newDef = {
                "type": "dateRange",
                "dateRange": f"{newStart}/{newEnd}",
            }
        elif shiftingDaysEnd is not None and type(shiftingDaysEnd) == int:
            newEnd = (end + datetime.timedelta(shiftingDaysEnd)).isoformat(
                timespec="milliseconds"
            )
            newDef = {
                "type": "dateRange",
                "dateRange": f"{start}/{newEnd}",
            }
        elif shiftingDaysStart is not None and type(shiftingDaysStart) == int:
            newStart = (start + datetime.timedelta(shiftingDaysStart)).isoformat(
                timespec="milliseconds"
            )
            newDef = {
                "type": "dateRange",
                "dateRange": f"{newStart}/{end}",
            }
        if pos > -1:
            self.__request["globalFilters"][pos] = newDef
        else:  ## in case there is no dateRange already
            self.__request["globalFilters"][pos].append(newDef)

    def removeGlobalFilter(self, index: int = None, filterId: str = None) -> None:
        """
        Remove a specific filter from the globalFilter list.
        You can use either the index of the list or the specific Id of the filter used.
        Arguments:
            index : REQUIRED : index in the list return
            filterId : REQUIRED : the id of the filter to be removed (ex: filterId, dateRange)
        """
        pos = -1
        if index is not None:
            del self.__request["globalFilters"][index]
        elif filterId is not None:
            for index, filter in enumerate(self.__request["globalFilters"]):
                if filterId in str(filter):
                    pos = index
            if pos > -1:
                del self.__request["globalFilters"][pos]
                ### decrementing the count for globalFilter
                self.__globalFiltersCount -= 1

    def to_dict(self) -> None:
        """
        Return the request definition
        """
        return deepcopy(self.__request)

    def save(self, fileName: str = None) -> None:
        """
        save the request definition in a JSON file.
        Argument:
            filename : OPTIONAL : Name of the file. (default cjapy_request_<timestamp>.json)
        """
        fileName = fileName or f"aa_request_{int(time())}.json"
        with open(fileName, "w") as f:
            f.write(json.dumps(self.to_dict(), indent=4))



import json
from copy import deepcopy
from aanalytics2 import config, connector
from typing import Union
from datetime import datetime,timedelta

class DataRepair:

    def __init__(self, 
                 company_id:str=None,
                 config_object: dict = config.config_object,
                 header: dict = config.header,
                 retry: int = 0,):
        """
        Instantiate the DataRepair class.
        Once instantiateed, the class will be connected to the Data Repair API and has generated a token.
        Arguments:
            company_id : REQUIRED : the company id to be used for the request.
            config_object : OPTIONAL : the configuration object to be used for the request.
            header : OPTIONAL : the header to be used for the request.
            retry : OPTIONAL : If you wish to retry failed GET requests
        
        """
        self.companyId = company_id
        self.endpoint = f"https://analytics.adobe.io/api/{company_id}/datarepair/v1"
        self.connector = connector.AdobeRequest(
            config_object=config_object, header=header, retry=retry,company_id=company_id)
        self.header = self.connector.header
        self.VARIABLES = {
            "activitymap": ["delete"],
            "campaign": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "entrypage": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "entrypageoriginal": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "evar1 - evar250": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "geodma": ["delete"],
            "geocity": ["delete"],
            "geocountry": ["delete"],
            "geolatitude":["delete"],
            "geolongitude": ["delete"],
            "georegion": ["delete"],
            "geozip": ["delete"],
            "ipaddress": ["delete"],
            "latitude": ["delete"],
            "longitude": ["delete"],
            "latlon1":["delete"],
            "latlon23":["delete"],
            "latlon45":["delete"],
            "pointofinterest":["delete"],
            "pointofinterestdistance": ["delete"],
            "mobileaction": ["delete"],
            "mobileappid":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilemessagebuttonname":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilemessageid":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaigncontent":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaignmedium":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaignsource":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaignterm":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaigntrackingcode":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "page": ["set","deleteQueryString","deleteQueryStringParameters"],
            "pageeventvar1": ["set","deleteQueryString","deleteQueryStringParameters"],
            "pageeventvar2": ["set","deleteQueryString","deleteQueryStringParameters"],
            "pageurl": ["deleteQueryString","deleteQueryStringParameters"],
            "pageurlfirsthit":["deleteQueryString","deleteQueryStringParameters"],
            "pageurlvisitstart": ["deleteQueryString","deleteQueryStringParameters"],
            "prop1 - prop75": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "referrer": ["deleteQueryString","deleteQueryStringParameters"],
            "referrerfirsthit": ["deleteQueryString","deleteQueryStringParameters"],
            "referrervisit": ["deleteQueryString","deleteQueryStringParameters"],
            "sitesections": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "video": ["set","deleteQueryString","deleteQueryStringParameters"],
            "videoad": ["set","deleteQueryString","deleteQueryStringParameters"],
            "videoadname": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoadplayername": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoadadvertiser": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudioalbum": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudioartist": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudioauthor": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudiolabel": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudiopublisher": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudiostation": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoadcampaign": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videochannel": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videochapter": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videocontenttype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoepisode": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videofeedtype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videomvpd": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoname": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videonetwork": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videopath": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoplayername": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoseason": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoshow": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoshowtype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videostreamtype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "zip": ["delete"]
            }

    
    def getEstimate(self,
                    dateStart:str=None,
                    dateEnd:str=None, 
                    rsid:str=None) -> dict:
        """
        Calculates the number of Server Calls for the given Report Suite and date range provided.
        It also returns a validationToken, which is required to use the Job endpoint.
        Arguments:
            dateStart: REQUIRED : The start date for the estimate in ISO format (YYYY-MM-DD).
            dateEnd: REQUIRED : The end date for the estimate in ISO format (YYYY-MM-DD).
            rsid: OPTIONAL : The report suite ID to be used for the estimate.
        Returns:
            dict: The estimate data as a dictionary.
        """
        if not dateStart or not dateEnd:
            raise ValueError("Both dateStart and dateEnd are required")
        
        params = {
            'dateRangeEnd': dateEnd,
            'dateRangeStart': dateStart
        }
        path = f"/{rsid}/serverCallEstimate"
        response = self.connector.getData(self.endpoint + path, params=params)
        return response
    
    def createJob(self,
        rsid:str=None,
        data:Union[dict,'DataRepairJobCreator',str]=None,
        token:str=None,
        dateStart:str=None,
        dateEnd:str=None,
    ) -> dict:
        """
        Create a job for data repair.
        Arguments:
            rsid: REQUIRED : The report suite ID to be used for the job.
            data: REQUIRED : DataRepairJobCreator instance or dictionary containing the job data or JSON file path.
            token : REQUIRED : The validation Token needed for the request
            dateStart: REQUIRED : The start date for the estimate in ISO format (YYYY-MM-DD).
            dateEnd: REQUIRED : The end date for the estimate in ISO format (YYYY-MM-DD).
        Returns:
            dict: The response from the API as a dictionary.
        """
        if rsid is None: 
            raise Exception('Need a ReportSuite ID')
        if data is None:
            raise Exception('Need a job description')
        if token is None:
            raise Exception('Need a token')
        params = {'validationToken':token,'dateRangeStart':dateStart,'dateRangeEnd':dateEnd}
        if isinstance(data, DataRepairJobCreator):
            data = data.to_dict()
        elif isinstance(data, dict):
            data = deepcopy(data)
        elif isinstance(data, str):
            if data.endswith('.json'):
                try:
                    with open(data, 'r') as f:
                        data = json.load(f)
                except FileNotFoundError as e:
                    raise ValueError(f"File not found: {e}")
                except json.JSONDecodeError as e:
                    raise ValueError(f"Invalid JSON file: {e}")
            else:
                raise ValueError("String data must be a path to a JSON file")
        else:
            raise TypeError("Data must be a DataRepairJobCreator instance or a dictionary")
        path = f"/{rsid}/job"
        response = self.connector.postData(self.endpoint + path, data=data,params=params)
        return response
    
    def getJobs(self, rsid: str = None) -> list:
        """
        Get all jobs for the given report suite ID.
        Arguments:
            rsid: REQUIRED : The report suite ID to be used for fetching jobs.
        Returns:
            list: the list of jobs.
        """
        path = f"/{rsid}/job"
        response = self.connector.getData(self.endpoint + path)
        return response
    
    def getJob(self, rsid: str = None, jobId: str = None) -> dict:
        """
        Get a specific job by its ID for the given report suite ID.
        Arguments:
            rsid: REQUIRED : The report suite ID to be used for fetching the job.
            jobId: REQUIRED : The job ID to be fetched.
        Returns:
            dict: The job data as a dictionary.
        """
        if not jobId:
            raise ValueError("jobId is required")
        path = f"/{rsid}/job/{jobId}"
        response = self.connector.getData(self.endpoint + path)
        return response
    
    def getUsage(self, dateStart:str=None, dateEnd:str=None,rsid: str = None) -> dict:
        """
        Get the usage for the given report suite ID.
        If no start and end dates are provided, it will return the usage for last 30 days.
        If only one of the dates is provided, it will take 30 days from the provided date
        Arguments:
            dateStart: REQUIRED : The start date for the usage in ISO format (YYYY-MM-DD).
            dateEnd: REQUIRED : The end date for the usage in ISO format (YYYY-MM-DD).
            rsid: OPTIONAL : If you want to see the usage for a single report suite ID.
        Returns:
            dict: The usage data as a dictionary.
        """
        if dateStart is None or dateEnd is None:
            dateStart = (datetime.now() - timedelta(days=30)).date().isoformat()
            dateEnd = datetime.now().date().isoformat()
        elif dateStart is None and dateEnd is not None:
            dateStart = (datetime.fromisoformat(dateEnd) - timedelta(days=30)).date().isoformat()
        elif dateStart is not None and dateEnd is None:
            dateEnd = (datetime.fromisoformat(dateStart) + timedelta(days=30)).date().isoformat()
        if rsid is None:
            path = f"/usage"
        else:
            path = f"/{rsid}/usage"
        params = {
            'dateRangeEnd': dateEnd,
            'dateRangeStart': dateStart
        }
        response = self.connector.getData(self.endpoint + path,params=params)
        return response


class DataRepairJobCreator:

    def __init__(self,data:Union[dict,str]=None):
        """
        Initialize the DataRepairJobCreator class.
        This class is used to create a job for data repair.
        Arguments:
            data: OPTIONAL : A dictionary or a JSON string containing the job data.
        """
        self.__data__ = {
            "variables":{

            }
        }
        if data is not None:
            if isinstance(data, str):
                if data.endswith('.json'):
                    try:
                        self.__data__ = json.loads(data)
                    except json.JSONDecodeError as e:
                        raise ValueError(f"Invalid JSON string: {e}")
            elif isinstance(data, dict):
                self.__data__ = deepcopy(data)
            else:
                raise TypeError("Data must be a dictionary or a JSON string")
        self.VARIABLES = {
            "activitymap": ["delete"],
            "campaign": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "entrypage": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "entrypageoriginal": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "evar1 - evar250": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "geodma": ["delete"],
            "geocity": ["delete"],
            "geocountry": ["delete"],
            "geolatitude":["delete"],
            "geolongitude": ["delete"],
            "georegion": ["delete"],
            "geozip": ["delete"],
            "ipaddress": ["delete"],
            "latitude": ["delete"],
            "longitude": ["delete"],
            "latlon1":["delete"],
            "latlon23":["delete"],
            "latlon45":["delete"],
            "pointofinterest":["delete"],
            "pointofinterestdistance": ["delete"],
            "mobileaction": ["delete"],
            "mobileappid":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilemessagebuttonname":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilemessageid":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaigncontent":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaignmedium":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaignsource":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaignterm":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "mobilerelaunchcampaigntrackingcode":["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "page": ["set","deleteQueryString","deleteQueryStringParameters"],
            "pageeventvar1": ["set","deleteQueryString","deleteQueryStringParameters"],
            "pageeventvar2": ["set","deleteQueryString","deleteQueryStringParameters"],
            "pageurl": ["deleteQueryString","deleteQueryStringParameters"],
            "pageurlfirsthit":["deleteQueryString","deleteQueryStringParameters"],
            "pageurlvisitstart": ["deleteQueryString","deleteQueryStringParameters"],
            "prop1 - prop75": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "referrer": ["deleteQueryString","deleteQueryStringParameters"],
            "referrerfirsthit": ["deleteQueryString","deleteQueryStringParameters"],
            "referrervisit": ["deleteQueryString","deleteQueryStringParameters"],
            "sitesections": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "video": ["set","deleteQueryString","deleteQueryStringParameters"],
            "videoad": ["set","deleteQueryString","deleteQueryStringParameters"],
            "videoadname": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoadplayername": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoadadvertiser": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudioalbum": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudioartist": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudioauthor": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudiolabel": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudiopublisher": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoaudiostation": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoadcampaign": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videochannel": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videochapter": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videocontenttype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoepisode": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videofeedtype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videomvpd": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoname": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videonetwork": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videopath": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoplayername": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoseason": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoshow": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videoshowtype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "videostreamtype": ["set","delete","deleteQueryString","deleteQueryStringParameters"],
            "zip": ["delete"]
            }
    
    def __str__(self):
        return json.dumps(self.__data__, indent=4)
    
    def __repr__(self):
        return json.dumps(self.__data__, indent=2)
    
    def addVariable(self,variableId:str=None,action:str=None,value:str=None,filterCondition:str=None,matchValues:Union[list,str]=None,filterVariable:str=None):
        """
        Add a variable to the job.
        Arguments:
            variableId: REQUIRED : The variable ID to be added.
            action: REQUIRED : The action to be performed. Choose from "set", "delete", "deleteQueryString", "deleteQueryStringParameters"
            value: REQUIRED if action is "set" : The value to be set for the variable.
            filterCondition: OPTIONAL : The filter condition to be applied. 
                Choose from "inList", "isEmpty", "contains", "doesNotContain", "startsWith",doesNotStartWith", "endsWith", "doesNotEndWith", "isURL", "isNotURL","isNumeric","isNotNumeric".
            matchValues: OPTIONAL : The list of values to be matched for the filter condition.
            filterVariable: OPTIONAL : The variable to be used for the filtering of the original variable.
        """
        var = {"action":action}
        if value:
            var['setValue'] = value
        if filterCondition:
            if 'filter' not in var.keys():
                var['filter'] = {}
            var['filter']['condition'] = filterCondition
        if matchValues:
            if 'filter' not in var.keys():
                var['filter'] = {}
            if isinstance(matchValues, str):
                var['filter']['matchValue'] = matchValues
            elif isinstance(matchValues, list):
                var['filter']['matchValues'] = matchValues
        if filterVariable:
            if 'filter' not in var.keys():
                var['filter'] = {}
            var['filter']['variable'] = filterVariable
        self.__data__['variables'][variableId] = deepcopy(var)
    
    def to_dict(self):
        """
        Convert the job data to a dictionary.
        Returns:
            dict: The job data as a dictionary.
        """
        return deepcopy(self.__data__)
    
    



import json
from copy import deepcopy
from typing import Union

class DwhCreator:

    def __init__(self, data: Union[str, dict] = None):
        """
        Initialize the Data Warehouse Creator with data.
        Arguments:
            data: OPTIONAL :  can be a JSON file path or a dictionary containing Data Warehouse data.
        """
        self.__data__ = {'request':{'reportParameters':{'dimensionList':[],'metricList':[],'segmentList':[]}},'delivery':{}}
        if isinstance(data, str):
            if data.endswith('.json'):
                with open(data, 'r') as file:
                    self.__data__ = json.load(file)
        elif isinstance(data, dict):
            self.__data__ = deepcopy(data)

    def __str__(self):
        return json.dumps(self.__data__, indent=4)
    
    def __repr__(self):
        return json.dumps(self.__data__, indent=2)
    
    def to_dict(self):
        """
        Convert the Data Warehouse data to a dictionary.
        Returns:
            dict: The Data Warehouse data as a dictionary.
        """
        return deepcopy(self.__data__)
    
    def setName(self, name: str):
        """
        Set the name for the Data Warehouse.
        Arguments:
            name: REQUIRED : Name to be used for the Data Warehouse.
        """
        if not name:
            raise ValueError("Name cannot be empty")
        self.__data__['request']['name'] = name
    
    def setRSID(self, rsid: str):
        """
        Set the report suite ID for the Data Warehouse.
        Arguments:
            rsid: REQUIRED : Report suite ID to be used for the Data Warehouse.
        """
        if not rsid:
            raise ValueError("Report suite ID cannot be empty")
        self.__data__['request']['rsid'] = rsid

    def setReportRange(self,preset:str=None,startDateTime: str=None, endDateTime: str=None):
        """
        Set the report range for the Data Warehouse.
        Arguments:
            preset: OPTIONAL : Preset for the report range, possible values: "Yesterday", "Today", "last_7_days", "last_30_days", "last_90_days", "last_365_days".
            startDateTime: REQUIRED : Start date and time in ISO format (YYYY-MM-DDTHH:MM:SSZ).
            endDateTime: REQUIRED : End date and time in ISO format (YYYY-MM-DDTHH:MM:SSZ).
        """
        if not startDateTime or not endDateTime:
            raise ValueError("Start and end date times cannot be empty")
        if preset:
            self.__data__['request']['reportParameters']['reportRange'] = {'preset': preset}
        else:       
            self.__data__['request']['reportParameters']['reportRange'] = {
                'startDateTime': startDateTime,
                'endDateTime': endDateTime
            }
    
    def addDimension(self, dimensionId: str,**kwargs):
        """
        Add a dimension to the Data Warehouse request.
        Arguments:
            dimensionId: REQUIRED : The ID of the dimension to be added.
                Example: "variables/geocity", "variables/prop1", "s0_000000000000000000000000"
        possible kwargs:
            additionalProperties
        """
        if not dimensionId:
            raise ValueError("Dimension ID cannot be empty")
        element = {'id': dimensionId}
        if dimensionId.startswith('s'):
            element['namespace'] = "Segment Service"
        if len(kwargs) > 0:
            element['properties'] = {}
            for key, value in kwargs.items():
                element['properties'][key] = value
        self.__data__['request']['reportParameters']['dimensionList'].append(element)

    def addMetric(self, metricId: str,**kwargs):
        """
        Add a metric to the Data Warehouse request.
        Arguments:
            metricId: REQUIRED : The ID of the metric to be added.
                Example: "metrics/visitors", "metrics/pageviews"
        possible kwargs:
            additionalProperties, ex: {"isParticipation": True}
        """
        if not metricId:
            raise ValueError("Metric ID cannot be empty")
        element = {'id': metricId}
        if len(kwargs) > 0:
            element['properties'] = {}
            for key, value in kwargs.items():
                element['properties'][key] = value
        self.__data__['request']['reportParameters']['metricList'].append(element)
    
    def addSegment(self, segmentId: str,**kwargs):
        """
        Add a segment to the Data Warehouse request.
        Arguments:
            segmentId: REQUIRED : The ID of the segment to be added.
                Example: "s1234567890abcdef"
        possible kwargs:
            additionalProperties
        """
        if not segmentId:
            raise ValueError("Segment ID cannot be empty")
        element = {'id': segmentId}
        if len(kwargs) > 0:
            element['properties'] = {}
            for key, value in kwargs.items():
                element['properties'][key] = value
        self.__data__['request']['reportParameters']['segmentList'].append(element)

    def addGranularity(self, granularity: str=None):
        """
        Set the granularity for the Data Warehouse request.
        Arguments:
            granularity: REQUIRED : Granularity of the report, possible values: "hourly", "daily", "weekly", "monthly", "yearly".
        """
        if not granularity:
            raise ValueError("Granularity cannot be empty")
        self.__data__['request']['reportParameters']['dateGranularity'] = granularity
    
    def setMaxNumberOfRows(self, maxRows: int):
        """
        Set the maximum number of rows for the Data Warehouse request.
        Arguments:
            maxRows: REQUIRED : Maximum number of rows to be returned in the report.
        """
        if maxRows <= 0:
            raise ValueError("Maximum number of rows must be greater than 0")
        self.__data__['request']['reportParameters']['numberOfRowsInTable'] = maxRows

    def setOutputFile(self, 
                      compression: str = None,
                      fileFormat: str = None,
                      sendEmptyFile: bool = True,
                      reportComments: str = None,
                      manifestFile: bool = True
                      ):
                      
        """
        Set the output format for the Data Warehouse request.
        Arguments:
            compression: OPTIONAL : Compression type for the output file, possible values: "gzip", "zip".
            fileFormat: OPTIONAL : Format of the output file, possible values: "csv", "tsv", "json".
            sendEmptyFile: OPTIONAL : Whether to send an empty file if no data is available (default True).
            reportComments: OPTIONAL : Comments to be included in the report.
            manifestFile: OPTIONAL : Whether to send a manifest file (default True).
        """
        self.__data__['request']['reportParameters']['outputFile'] = {}
        if compression:
            self.__data__['request']['reportParameters']['outputFile']['compressionFormat'] = compression
        if fileFormat:
            self.__data__['request']['reportParameters']['outputFile']['fileFormat'] = fileFormat
        self.__data__['request']['reportParameters']['outputFile']['sendEmptyFileForNoDataReport'] = sendEmptyFile
        if reportComments:
            self.__data__['request']['reportParameters']['outputFile']['beginningOfReportComments'] = reportComments
        self.__data__['request']['reportParameters']['outputFile']['sendManifestFile'] = manifestFile


    def setSchedule(self, 
                    frequency:str=None,
                    every:int=None,
                    dayOfMonth:int=None,
                    dayOfWeek:str=None,
                    month:str=None,
                    weekOfMonth:str=None,
                    cancelMethod:str=None,
                    end_occurrences:int=None,
                    scheduleAt:str=None,
                    cancelDate:str=None):
        """
        Set the schedule for the Data Warehouse.
        Arguments:
            frequency: OPTIONAL : Frequency of the schedule, possible values: "hourly", "daily", "weekly", "monthly", "yearly".
            every: OPTIONAL : Interval for the frequency, e.g., every 2 days.
            dayOfMonth: OPTIONAL : Day of the month for the schedule (1-31).
            dayOfWeek: OPTIONAL : Day of the week for the schedule (0-6, where 0 is Sunday).
            month: OPTIONAL : Month for the schedule (1-12).
            weekOfMonth: OPTIONAL : Week of the month for the schedule (1-5).
            cancelMethod: OPTIONAL : Method to cancel the schedule, possible values "afterOccurrences", "end_of_period".
            occurrences: OPTIONAL : Number of occurrences for the schedule before it ends.
            scheduleAt : OPTIONAL : Timestamp of when the request is scheduled.
            cancelDate : OPTIONAL : Date when the schedule should be cancelled.
        """
        schedule = {"periodSettings":{},"cancelSettings":{}}
        if frequency is not None:
            schedule['periodSettings']['frequency'] = frequency
        if every is not None:
            schedule['periodSettings']['every'] = every
        if dayOfMonth is not None:
            schedule['periodSettings']['dayOfMonth'] = dayOfMonth
        if dayOfWeek is not None:
            schedule['periodSettings']['dayOfWeek'] = dayOfWeek
        if month is not None:
            schedule['periodSettings']['month'] = month
        if weekOfMonth is not None:
            schedule['periodSettings']['weekOfMonth'] = weekOfMonth
        if scheduleAt is not None:
            schedule['periodSettings']['scheduleAt'] = scheduleAt
        if cancelMethod:
            schedule['cancelSettings']['cancelMethod'] = cancelMethod
        if end_occurrences is not None:
            schedule['cancelSettings']['endAfterNumOccurrences']  = end_occurrences
        if cancelDate is not None:
            schedule['cancelSettings']['cancelDate'] = cancelDate
        self.__data__['schedule'] = schedule
    
    def setDelivery(self,
                    exportLocationUUID: str = None,
                    emailNotificationTo: str = False,
                    emailNotificationFrom: str = None,
                    emailNotificationSubject: str = None,
                    emailNotificationNote: str = None,
                    legacyEmail: bool = True,
                    legacyFTP_username: str = None,
                    legacyFTP_host: str = None,
                    legacyFTP_port: int = None,
                    legacyFTP_directory: str = None,
                    legacyAzure_account:str=None,
                    legacyAzure_container:str=None,
                    legacyAzure_prefix:str=None,
                    leggacyS3_bucket:str=None,
                    legacyS3_accessKey:str=None,
                    legacyS3_awsPath:str=None,
                    ):
        """
        Set the delivery method for the Data Warehouse request.
        Arguments:
            exportLocationUUID: OPTIONAL : UUID of the export location.
            emailNotificationTo: OPTIONAL : Email address to send notifications to.
            emailNotificationFrom: OPTIONAL : Email address to send notifications from.
            emailNotificationSubject: OPTIONAL : Subject of the email notification.
            emailNotificationNote: OPTIONAL : Note to include in the email notification.
            legacyEmail: OPTIONAL : Whether to use legacy email delivery (default True).
            legacyFTP_username: OPTIONAL : Username for legacy FTP delivery.
            legacyFTP_host: OPTIONAL : Host for legacy FTP delivery.
            legacyFTP_port: OPTIONAL : Port for legacy FTP delivery.
            legacyFTP_directory: OPTIONAL : Directory for legacy FTP delivery.
            legacyAzure_account: OPTIONAL : Account for legacy Azure delivery.
            legacyAzure_container: OPTIONAL : Container for legacy Azure delivery.
            legacyAzure_prefix: OPTIONAL : Prefix for legacy Azure delivery.
            leggacyS3_bucket: OPTIONAL : Bucket for legacy S3 delivery.
            legacyS3_accessKey: OPTIONAL : Access key for legacy S3 delivery.
            legacyS3_awsPath: OPTIONAL : AWS path for legacy S3 delivery.
        """
        delivery = {}
        if exportLocationUUID:
            delivery['exportLocationUUID'] = exportLocationUUID
        if emailNotificationTo:
            if 'email' not in delivery.keys():
                delivery['email'] = {}
            delivery['email']['notificationEmailTo'] = emailNotificationTo
            if emailNotificationFrom:
                delivery['email']['notificationEmailFrom'] = emailNotificationFrom
            if emailNotificationSubject:
                delivery['email']['notificationEmailSubject'] = emailNotificationSubject
            if emailNotificationNote:
                delivery['email']['notificationEmailNotes'] = emailNotificationNote
        if legacyEmail:
            delivery['legacyEmail'] = True
        if legacyFTP_username:
            if 'legacyFTP' not in delivery.keys():
                delivery['legacyFTP'] = {}
            delivery['legacyFTP']['username'] = legacyFTP_username
            if legacyFTP_host:
                delivery['legacyFTP']['host'] = legacyFTP_host
            if legacyFTP_port:
                delivery['legacyFTP']['port'] = legacyFTP_port
            if legacyFTP_directory:
                delivery['legacyFTP']['directory'] = legacyFTP_directory
        if legacyAzure_account:
            if 'legacyAzure' not in delivery.keys():
                delivery['legacyAzure'] = {}
            delivery['legacyAzure']['account'] = legacyAzure_account
            if legacyAzure_container:
                delivery['legacyAzure']['container'] = legacyAzure_container
            if legacyAzure_prefix:
                delivery['legacyAzure']['prefix'] = legacyAzure_prefix
        if leggacyS3_bucket:
            if 'legacyS3' not in delivery.keys():
                delivery['legacyS3'] = {}
            delivery['legacyS3']['bucket'] = leggacyS3_bucket
            if legacyS3_accessKey:
                delivery['legacyS3']['accessKey'] = legacyS3_accessKey
            if legacyS3_awsPath:
                delivery['legacyS3']['awsPath'] = legacyS3_awsPath
        self.__data__['delivery'] = delivery

    